local DROP_MAG_HOLD_T = 1/3
-- Holding the "reload" button for this duration will cause you to perform a reload
-- with increased speed, at the cost of wasting your remaining ammo in the magazine
-- Tapping the reload button for any duration less than this will cause a normal reload

local DROPMAG_RELOAD_SPEED_MUL = 2
--bonus reload speed multiplier on activating dropmag

--creates and uses following vars:
--PlayerStandard._check_dropmag_t
	--[number] time started holding reload button;
	--must hold longer than HOLD_T to activate fast-reload "drop mag" mechanic
	--if released before HOLD_T has elapsed, do normal reload
--PlayerStandard._dropmag_reload_activated 
	--[bool] flag indicating whether dropmag input was performed; 
	--used to avoid having to actually override _update_reload_timers()

local orig_check_reload = PlayerStandard._check_action_reload
function PlayerStandard:_check_action_reload(t, input,...)
	local weapon = self._equipped_unit and self._equipped_unit:base()
		
	if not weapon or weapon._use_shotgun_reload then 
		return orig_check_reload(self,t,input,...)
	end


	local action_wanted = input.btn_reload_press or self._controller:get_input_bool("reload")
	if self:_is_reloading() or self:_changing_weapon() or self:_is_meleeing() or self._use_item_expire_t or self:_interacting() or self:_is_throwing_projectile() or weapon:clip_full() then 
		return 
	end
	
	if self._controller:get_input_released("reload") then
		if not self._check_dropmag_t then 
			input.btn_reload_press = true
			return orig_check_reload(self,t,input,...)
		end
		
		if (t - self._check_dropmag_t) >= DROP_MAG_HOLD_T then 
			--start dropmag
			self:_start_action_dropmag_reload_enter(t)
			return true
		else
			input.btn_reload_press = true
			--do normal reload
		end		
		self._check_dropmag_t = nil
	elseif action_wanted then 
		if self._check_dropmag_t and (t - self._check_dropmag_t >= DROP_MAG_HOLD_T) then 
			--start dropmag from held sufficiently long
			self:_start_action_dropmag_reload_enter(t)
			self._check_dropmag_t = nil
			return true
		else
			self._check_dropmag_t = self._check_dropmag_t or t --start timer	
		end
		return
	end
	
	return orig_check_reload(self,t,input,...)
end

function PlayerStandard:_start_action_dropmag_reload_enter(t,...)
	local weapon = self._equipped_unit:base() 
	
	
	if weapon and weapon:can_reload() then 
		if weapon._use_shotgun_reload then 
			return self:_start_action_reload(t,...)
		end
		self._dropmag_reload_activated = true
		
		weapon:tweak_data_anim_stop("fire")

		local speed_multiplier = weapon:reload_speed_multiplier() * DROPMAG_RELOAD_SPEED_MUL --this is the only significant change 
		local tweak_data = weapon:weapon_tweak_data()
		local reload_anim = "reload"
		local reload_prefix = weapon:reload_prefix() or ""
		local reload_name_id = tweak_data.animations.reload_name_id or weapon.name_id

		if weapon:clip_empty() then
			local reload_ids = Idstring(reload_prefix .. "reload_" .. reload_name_id)
			local result = self._ext_camera:play_redirect(reload_ids, speed_multiplier)

--			Application:trace("PlayerStandard:_start_action_reload( t ): ", reload_ids)

			self._state_data.reload_expire_t = t + (tweak_data.timers.reload_empty or weapon:reload_expire_t() or 2.6) / speed_multiplier
		else
			reload_anim = "reload_not_empty"
			local reload_ids = Idstring(reload_prefix .. "reload_not_empty_" .. reload_name_id)
			local result = self._ext_camera:play_redirect(reload_ids, speed_multiplier)

--			Application:trace("PlayerStandard:_start_action_reload( t ): ", reload_ids)

			self._state_data.reload_expire_t = t + (tweak_data.timers.reload_not_empty or weapon:reload_expire_t() or 2.2) / speed_multiplier
		end

		weapon:start_reload()

		if not weapon:tweak_data_anim_play(reload_anim, speed_multiplier) then
			weapon:tweak_data_anim_play("reload", speed_multiplier)
--			Application:trace("PlayerStandard:_start_action_reload( t ): ", reload_anim)
		end

		self._ext_network:send("reload_weapon", empty_reload, speed_multiplier)
	end
end

local orig_update_reload = PlayerStandard._update_reload_timers
function PlayerStandard:_update_reload_timers(t, dt, input,...)
	if not self._dropmag_reload_activated then 
		return orig_update_reload(self,t,dt,input,...)
	end
	
	if self._state_data.reload_enter_expire_t and self._state_data.reload_enter_expire_t <= t then
		self._state_data.reload_enter_expire_t = nil

		self:_start_action_reload(t)
	end

	if self._state_data.reload_expire_t then
		local interupt = nil

		if self._equipped_unit:base():update_reloading(t, dt, self._state_data.reload_expire_t - t) then
			managers.hud:set_ammo_amount(self._equipped_unit:base():selection_index(), self._equipped_unit:base():ammo_info())

			if self._queue_reload_interupt then
				self._queue_reload_interupt = nil
				interupt = true
			end
		end

		if self._state_data.reload_expire_t <= t or interupt then
			managers.player:remove_property("shock_and_awe_reload_multiplier")

			self._state_data.reload_expire_t = nil

			if self._equipped_unit:base():reload_exit_expire_t() then
				local speed_multiplier = self._equipped_unit:base():reload_speed_multiplier() * DROPMAG_RELOAD_SPEED_MUL --changed this

				if self._equipped_unit:base():started_reload_empty() then
					self._state_data.reload_exit_expire_t = t + self._equipped_unit:base():reload_exit_expire_t() / speed_multiplier

					self._ext_camera:play_redirect(self:get_animation("reload_exit"), speed_multiplier)
					self._equipped_unit:base():tweak_data_anim_play("reload_exit", speed_multiplier)
				else
					self._state_data.reload_exit_expire_t = t + self._equipped_unit:base():reload_not_empty_exit_expire_t() / speed_multiplier

					self._ext_camera:play_redirect(self:get_animation("reload_not_empty_exit"), speed_multiplier)
					self._equipped_unit:base():tweak_data_anim_play("reload_not_empty_exit", speed_multiplier)
				end
			elseif self._equipped_unit then
				if not interupt then
					self._equipped_unit:base():on_reload(nil,true) --changed this
					self._dropmag_reload_activated = nil
				end

				managers.statistics:reloaded()
				managers.hud:set_ammo_amount(self._equipped_unit:base():selection_index(), self._equipped_unit:base():ammo_info())

				if input.btn_steelsight_state then
					self._steelsight_wanted = true
				elseif self.RUN_AND_RELOAD and self._running and not self._end_running_expire_t and not self._equipped_unit:base():run_and_shoot_allowed() then
					self._ext_camera:play_redirect(self:get_animation("start_running"))
				end
			end
		end
	end

	if self._state_data.reload_exit_expire_t and self._state_data.reload_exit_expire_t <= t then
		self._state_data.reload_exit_expire_t = nil

		if self._equipped_unit then
			managers.statistics:reloaded()
			managers.hud:set_ammo_amount(self._equipped_unit:base():selection_index(), self._equipped_unit:base():ammo_info())

			if input.btn_steelsight_state then
				self._steelsight_wanted = true
			elseif self.RUN_AND_RELOAD and self._running and not self._end_running_expire_t and not self._equipped_unit:base():run_and_shoot_allowed() then
				self._ext_camera:play_redirect(self:get_animation("start_running"))
			end

			if self._equipped_unit:base().on_reload_stop then
				self._equipped_unit:base():on_reload_stop()
			end
		end
	end
end

Hooks:PreHook(PlayerStandard,"_interrupt_action_reload","interrupt_dropmag_reload",function(self,t)
	self._check_dropmag_t = nil
	self._dropmag_reload_activated = nil
end)

